/*
 * Copyright (C) 2000-2001 Damien Douxchamps  <douxchamps@ieee.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "definitions.h"
#include "connect_signals.h"
#include <libdc1394/dc1394_control.h>
#include <libraw1394/raw1394.h>
#include "video1394.h"

extern GtkWidget *commander_window;
extern GtkWidget *porthole_window;
extern GtkWidget *color_window;
extern GtkWidget *status_window;
extern GtkWidget *aperture_window;
extern GtkWidget *configure_window;
extern GtkWidget *capture_window;
extern GtkWidget *temperature_window;
extern dc1394_camerainfo *camera;
extern dc1394_feature_set *feature_set;
extern char* feature_op_list[NUM_FEATURES];
extern char* feature_list[NUM_FEATURES];
extern char* feature_frame_list[NUM_FEATURES];
extern char* feature_scale_list[NUM_FEATURES];
extern char* feature_man_list[NUM_FEATURES];
extern char* feature_auto_list[NUM_FEATURES];
extern char* feature_power_list[NUM_FEATURES];



void
ConnectRangeSignals()
{
  GtkAdjustment* adj;

  // commander window:
  adj=gtk_spin_button_get_adjustment( (GtkSpinButton*) lookup_widget(commander_window,"trigger_count"));
  gtk_signal_connect (GTK_OBJECT (adj), "value_changed", GTK_SIGNAL_FUNC (on_scale_value_changed), (int*) FEATURE_TRIGGER);

  // porthole window:
  adj=gtk_range_get_adjustment( (GtkRange*) lookup_widget(porthole_window,"zoom_scale"));
  gtk_signal_connect (GTK_OBJECT (adj), "value_changed", GTK_SIGNAL_FUNC (on_scale_value_changed), (int*) FEATURE_ZOOM);
  adj=gtk_range_get_adjustment( (GtkRange*) lookup_widget(porthole_window,"pan_scale"));
  gtk_signal_connect (GTK_OBJECT (adj), "value_changed", GTK_SIGNAL_FUNC (on_scale_value_changed), (int*) FEATURE_PAN);
  adj=gtk_range_get_adjustment( (GtkRange*) lookup_widget(porthole_window,"tilt_scale"));
  gtk_signal_connect (GTK_OBJECT (adj), "value_changed", GTK_SIGNAL_FUNC (on_scale_value_changed), (int*) FEATURE_TILT);
  adj=gtk_range_get_adjustment( (GtkRange*) lookup_widget(porthole_window,"focus_scale"));
  gtk_signal_connect (GTK_OBJECT (adj), "value_changed", GTK_SIGNAL_FUNC (on_scale_value_changed), (int*) FEATURE_FOCUS);

  // color window:
  adj=gtk_range_get_adjustment( (GtkRange*) lookup_widget(color_window,"brightness_scale"));
  gtk_signal_connect (GTK_OBJECT (adj), "value_changed", GTK_SIGNAL_FUNC (on_scale_value_changed), (int*) FEATURE_BRIGHTNESS);
  adj=gtk_range_get_adjustment( (GtkRange*) lookup_widget(color_window,"gamma_scale"));
  gtk_signal_connect (GTK_OBJECT (adj), "value_changed", GTK_SIGNAL_FUNC (on_scale_value_changed), (int*) FEATURE_GAMMA);
  adj=gtk_range_get_adjustment( (GtkRange*) lookup_widget(color_window,"saturation_scale"));
  gtk_signal_connect (GTK_OBJECT (adj), "value_changed", GTK_SIGNAL_FUNC (on_scale_value_changed), (int*) FEATURE_SATURATION);
  adj=gtk_range_get_adjustment( (GtkRange*) lookup_widget(color_window,"hue_scale"));
  gtk_signal_connect (GTK_OBJECT (adj), "value_changed", GTK_SIGNAL_FUNC (on_scale_value_changed), (int*) FEATURE_HUE);
  adj=gtk_range_get_adjustment( (GtkRange*) lookup_widget(color_window,"sharpness_scale"));
  gtk_signal_connect (GTK_OBJECT (adj), "value_changed", GTK_SIGNAL_FUNC (on_scale_value_changed), (int*) FEATURE_SHARPNESS);
  adj=gtk_range_get_adjustment( (GtkRange*) lookup_widget(color_window,"whitebal_BU_scale"));
  gtk_signal_connect (GTK_OBJECT (adj), "value_changed", GTK_SIGNAL_FUNC (on_scale_value_changed), (int*) FEATURE_WHITE_BALANCE+BU);
  adj=gtk_range_get_adjustment( (GtkRange*) lookup_widget(color_window,"whitebal_RV_scale"));
  gtk_signal_connect (GTK_OBJECT (adj), "value_changed", GTK_SIGNAL_FUNC (on_scale_value_changed), (int*) FEATURE_WHITE_BALANCE+RV);

  // aperture_window:
  adj=gtk_range_get_adjustment( (GtkRange*) lookup_widget(aperture_window,"exposure_scale"));
  gtk_signal_connect (GTK_OBJECT (adj), "value_changed", GTK_SIGNAL_FUNC (on_scale_value_changed), (int*) FEATURE_EXPOSURE);
  adj=gtk_range_get_adjustment( (GtkRange*) lookup_widget(aperture_window,"iris_scale"));
  gtk_signal_connect (GTK_OBJECT (adj), "value_changed", GTK_SIGNAL_FUNC (on_scale_value_changed), (int*) FEATURE_IRIS);
  adj=gtk_range_get_adjustment( (GtkRange*) lookup_widget(aperture_window,"shutter_scale"));
  gtk_signal_connect (GTK_OBJECT (adj), "value_changed", GTK_SIGNAL_FUNC (on_scale_value_changed), (int*) FEATURE_SHUTTER);
  adj=gtk_range_get_adjustment( (GtkRange*) lookup_widget(aperture_window,"gain_scale"));
  gtk_signal_connect (GTK_OBJECT (adj), "value_changed", GTK_SIGNAL_FUNC (on_scale_value_changed), (int*) FEATURE_GAIN);
  adj=gtk_range_get_adjustment( (GtkRange*) lookup_widget(aperture_window,"filter_scale"));
  gtk_signal_connect (GTK_OBJECT (adj), "value_changed", GTK_SIGNAL_FUNC (on_scale_value_changed), (int*) FEATURE_OPTICAL_FILTER);

  // capture window:
  adj=gtk_range_get_adjustment( (GtkRange*) lookup_widget(capture_window,"capqual_scale"));
  gtk_signal_connect (GTK_OBJECT (adj), "value_changed", GTK_SIGNAL_FUNC (on_scale_value_changed), (int*) FEATURE_CAPTURE_QUALITY);
  adj=gtk_range_get_adjustment( (GtkRange*) lookup_widget(capture_window,"capsize_scale"));
  gtk_signal_connect (GTK_OBJECT (adj), "value_changed", GTK_SIGNAL_FUNC (on_scale_value_changed), (int*) FEATURE_CAPTURE_SIZE);

  // temperature window:
  adj=gtk_range_get_adjustment( (GtkRange*) lookup_widget(temperature_window,"temp_goal_scale"));
  gtk_signal_connect (GTK_OBJECT (adj), "value_changed", GTK_SIGNAL_FUNC (on_scale_value_changed), (int*) FEATURE_TEMPERATURE);

}

void
on_scale_value_changed             ( GtkAdjustment    *adj,
				     gpointer         user_data)
{
  int err, bu, rv;

  //printf("User data: %d\n",(int)(int*)(int)user_data);
  switch((int)user_data)
    {
      //printf("User data: %d\n",(int)user_data);
      case FEATURE_TEMPERATURE:
	err=dc1394_set_temperature(camera->handle,camera->id,adj->value);
	feature_set->feature[FEATURE_TEMPERATURE].target_value=adj->value; break;
      case FEATURE_WHITE_BALANCE+BU*4: // why oh why is there a *4?
	//printf("Writing BU\n");
	err=dc1394_get_white_balance(camera->handle,camera->id,&bu, &rv);
	err=dc1394_set_white_balance(camera->handle,camera->id,adj->value, rv);
	feature_set->feature[FEATURE_WHITE_BALANCE].BU_value=adj->value; break;
      case FEATURE_WHITE_BALANCE+RV*4: // why oh why is there a *4?
	//printf("Writing RV\n");
	err=dc1394_get_white_balance(camera->handle,camera->id,&bu, &rv);
	err=dc1394_set_white_balance(camera->handle,camera->id,bu, adj->value);
	feature_set->feature[FEATURE_WHITE_BALANCE].RV_value=adj->value; break;
      default: // includes trigger_count
	err=dc1394_set_feature_value(camera->handle,camera->id,(int)user_data,adj->value);
	//printf("Writing default: error=%d\n",err);
	feature_set->feature[(int)user_data].value=adj->value;
    }
}
